#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
File: __init__.py
Description: Python SDK of the Sparshik KYC API.
"""

from . import applicants
from . import documents
from . import challenge
from . import mask
from . import reports
from . import utils

from .utils import KYCException
from .utils import Key
from .utils import BaseUrl



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
File: utils.py
Description: Shared utilities for the Python SDK of the Sparshik KYC API.
"""
import os.path
import requests

DEFAULT_BASE_URL = os.environ.get('KYC_ENDPOINT', "https://api.kyc.sparshik.com")


class KYCException(Exception):
	"""Custom Exception for the python SDK of the Sparshik KYC API.
	Attributes:
		status_code: HTTP response status code.
		type: error type.
		message: error message string
	"""

	def __init__(self, status_code, type, message):
		super(KYCException, self).__init__()
		self.status_code = status_code
		self.type = type
		self.message = message

	def __str__(self):
		return ('Error when calling Sparshik KYC API:\n'
				'\tstatus_code: {}\n'
				'\ttype: {}\n'
				'\tmessage: {}\n').format(self.status_code, self.type, self.message)


class Key(object):
	"""Manage API Token Key."""

	@classmethod
	def set(cls, Token):
		"""Set the API Token Key."""
		cls.Token = Token

	@classmethod
	def get(cls):
		"""Get the API Token Key."""
		if not hasattr(cls, 'Token'):
			cls.Token = None
		return cls.Token


class BaseUrl(object):
	"""Manage Base URL Key."""

	@classmethod
	def set(cls, base_url):
		"""Set the Base URL."""
		if not base_url.endswith('/'):
			base_url += '/'
		cls.base_url = base_url

	@classmethod
	def get(cls):
		"""Get the Base URL."""
		if not hasattr(cls, 'base_url') or not cls.base_url:
			cls.base_url = DEFAULT_BASE_URL
		return cls.base_url


def request(method, url, files=None, json=None, data=None, headers=None):
	""" Interface for request.
	Args:
		method : Type of HTTP method - GET, POST, DEL or PUT
		url : url for different components
		files : File path or Image path or Data URL
		json : Dict including parameters required for API call

	Returns:
		returns HTTP status code.
	"""

	"""Universal interface for request."""

	# Make it possible to call only with short name (without BaseUrl).
	if not url.startswith('https://'):
		url = BaseUrl.get() + url

	# Setup the headers with default Content-Type and Subscription Key.
	headers = headers or {}
	headers['Authorization'] = "Token {}".format(Key.get())

	if files is not None and len(files.keys()) == 0:
		if data is not None:
			json = data
			data = None

	response = requests.request(method, data=data, json=json, url=url, files=files, headers=headers)

	# Handle result and raise custom exception when something wrong.

	if response.status_code not in [200, 201, 202, 204]:
		try:
			error_msg = response.json()["error"]
		except:
			raise KYCException(response.status_code, response.status_code, response.text)

		raise KYCException(response.status_code, error_msg["type"], error_msg["message"])

	# Prevent `response.json()` complains about empty response.
	if response.text:
		result = response.json()
	else:
		result = dict()

	return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
File: documents.py
Description: Documents section of the Sparshik KYC API.
"""

from .utils import request


def parse_doc(file_path, attributes=None, doc_type=None, user_request_id=None, image_crops=None, delete_at=None,
			  image_url=None, data_uri=None):
	""" Identify or Parse KYC Documents such as Aadhaar, Pan, Passport and Voter ID.
	Args:
		file_path : Path to File or Image or Data URL
		attributes : [Optional] type to identify document type, mask to mask number in aadhaar. 
			Supported attribute include type,mask
		doc_type : [Optional] Supported doc-type include aadhaar, pan, or passport
		user_request_id : [Optional] user request id, supplied with response 
		image_crops : [Optional] image crops of original document. 
			Supported image_crops include front, front_top, front_bottom, back, qr_code, photo
		delete_at : [Optional] when to delete record, Ex, 14-07-2020
		image_url : [Optional] data_uri or image_url or file_path ( one of them is required)
		data_uri : [Optional] data_uri or image_url or file_path ( one of them is required)
	
	Returns:
		returns parsed information and HTTP status code.
	"""

	url = "documents/"

	if file_path:
		files = {'file_path': open(file_path, 'rb')}
	else:
		files = {}

	# create only one data field dict
	json = {'doc_type': doc_type, 'attributes': attributes, 'user_request_id': user_request_id,
			'image_crops': image_crops, 'delete_at': delete_at}

	if image_url:
		json["image_url"] = image_url
	elif data_uri:
		json["data_uri"] = data_uri

	# call internal abstract API function
	response = request('POST', url, files=files, json=json)
	return response


def offline_aadhaar(file_path, share_code, user_request_id=None, image_url=None, data_uri=None):
	""" Verify Aadhaar offline with Share code.
	Args:
		file_path : Path to File or Image or Data URL
		share_code : 4 digit share code.
		user_request_id : [Optional] user request id, supplied with response
		image_url : [Optional] data_uri or image_url or file_path ( one of them is required)
		data_uri : [Optional] data_uri or image_url or file_path ( one of them is required)
	
	Returns:
		returns HTTP status code.
	"""

	url = "documents/offline"

	if file_path:
		files = {'file_path': open(file_path, 'rb')}
	else:
		files = {}

	# create only one data field dict
	json = {'share_code': share_code, 'user_request_id': user_request_id}

	if image_url:
		json["image_url"] = image_url
	elif data_uri:
		json["data_uri"] = data_uri

	# call internal abstract API function
	response = request('POST', url, files=files, data=json)
	return response

